@@ -1,6 +1,5 @@
-# pyformat: mode=midnight
-# ==============================================================================
 # Copyright 2022 Intrinsic Innovation LLC.
+# Copyright 2023 DeepMind Technologies Limited.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +12,13 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-# ==============================================================================
+
 """Defines a logical factor."""
 
 import dataclasses
 import functools
-from typing import List, Mapping, Optional, Sequence, Tuple, Union
+from typing import Any, Dict, Hashable, List, Mapping, Optional, Sequence, Tuple, Union
+import warnings
 
 import jax
 from jax.nn import log_sigmoid
@@ -38,21 +38,21 @@ class LogicalWiring(factor.Wiring):
 
   Attributes:
     parents_edge_states: Array of shape (num_parents, 2)
-        parents_edge_states[ii, 0] contains the global LogicalFactor index
-        parents_edge_states[ii, 1] contains the message index of the parent
-        variable's relevant state
-
-      Both indices only take into account the LogicalFactors of the same subtype
-      (OR/AND) of the FactorGraph
+      parents_edge_states[ii, 0] contains the global LogicalFactor index
+      parents_edge_states[ii, 1] contains the message index of the parent
+      variable's relevant state
       The message index of the parent variable's other state is
       parents_edge_states[ii, 1] + edge_states_offset
+      Both indices only take into account the LogicalFactors of the same subtype
+      (OR/AND) of the FactorGraph
 
     children_edge_states: Array of shape (num_factors,)
       children_edge_states[ii] contains the message index of the child
-      variable's relevant state , which takes into account all the
-      LogicalFactors of the same subtype (OR/AND) of the FactorGraph
+      variable's relevant state
       The message index of the child variable's other state is
-      children_edge_states[ii, 1] + edge_states_offset
+      children_edge_states[ii] + edge_states_offset
+      Only takes into account the LogicalFactors of the same subtype (OR/AND)
+      of the FactorGraph
 
     edge_states_offset: Offset to go from a variable's relevant state to its
       other state
@@ -94,16 +94,25 @@ class LogicalWiring(factor.Wiring):
             f" (for AND), but is {self.edge_states_offset}"
         )
 
+  def get_inference_arguments(self) -> Dict[str, Any]:
+    """Return the list of arguments to run BP with LogicalWirings."""
+    return {
+        "parents_factor_indices": self.parents_edge_states[..., 0],
+        "parents_msg_indices": self.parents_edge_states[..., 1],
+        "children_edge_states": self.children_edge_states,
+        "edge_states_offset": self.edge_states_offset,
+    }
+
 
 @dataclasses.dataclass(frozen=True, eq=False)
 class LogicalFactor(factor.Factor):
   """A logical OR/AND factor of the form (p1,...,pn, c) where p1,...,pn are the parents variables and c is the child variable.
 
   Attributes:
-      edge_states_offset: Offset to go from a variable's relevant state to its
-        other state
-        For ORFactors the edge_states_offset is 1
-        For ANDFactors the edge_states_offset is -1
+    edge_states_offset: Offset to go from a variable's relevant state to its
+      other state
+      For ORFactors the edge_states_offset is 1
+      For ANDFactors the edge_states_offset is -1
 
   Raises:
     ValueError: If:
@@ -139,8 +148,7 @@ class LogicalFactor(factor.Factor):
     """
     if not wirings:
       return LogicalWiring(
-          edges_num_states=np.empty((0,), dtype=int),
-          var_states_for_edges=np.empty((0,), dtype=int),
+          var_states_for_edges=np.empty((0, 3), dtype=int),
           parents_edge_states=np.empty((0, 2), dtype=int),
           children_edge_states=np.empty((0,), dtype=int),
           edge_states_offset=1,
@@ -148,9 +156,23 @@ class LogicalFactor(factor.Factor):
 
     # Factors indices offsets
     num_factors_cumsum = np.insert(
-        np.array(
+        np.cumsum(
             [wiring.parents_edge_states[-1, 0] + 1 for wiring in wirings]
-        ).cumsum(),
+        ),
+        0,
+        0,
+    )[:-1]
+
+    # Messages offsets
+    concatenated_var_states_for_edges = factor.concatenate_var_states_for_edges(
+        [wiring.var_states_for_edges for wiring in wirings]
+    )
+
+    # Factors indices offsets
+    num_factors_cumsum = np.insert(
+        np.cumsum(
+            [wiring.parents_edge_states[-1, 0] + 1 for wiring in wirings]
+        ),
         0,
         0,
     )[:-1]
@@ -158,9 +180,9 @@ class LogicalFactor(factor.Factor):
     # Messages offsets
     # Note: this is all the factor_to_msgs_starts for the LogicalFactors
     num_edge_states_cumsum = np.insert(
-        np.array(
-            [wiring.edges_num_states.sum() for wiring in wirings]
-        ).cumsum(),
+        np.cumsum(
+            [wiring.var_states_for_edges.shape[0] for wiring in wirings]
+        ),
         0,
         0,
     )[:-1]
@@ -177,12 +199,7 @@ class LogicalFactor(factor.Factor):
       )
 
     return LogicalWiring(
-        edges_num_states=np.concatenate(
-            [wiring.edges_num_states for wiring in wirings]
-        ),
-        var_states_for_edges=np.concatenate(
-            [wiring.var_states_for_edges for wiring in wirings]
-        ),
+        var_states_for_edges=concatenated_var_states_for_edges,
         parents_edge_states=np.concatenate(parents_edge_states, axis=0),
         children_edge_states=np.concatenate(children_edge_states, axis=0),
         edge_states_offset=wirings[0].edge_states_offset,
@@ -205,65 +222,141 @@ class LogicalFactor(factor.Factor):
       factor_edges_num_states: An array concatenating the number of states for
         the variables connected to each Factor of the FactorGroup. Each variable
         will appear once for each Factor it connects to.
-
       variables_for_factors: A tuple of tuples containing variables connected to
         each Factor of the FactorGroup. Each variable will appear once for each
         Factor it connects to.
-
       factor_sizes: An array containing the different factor sizes.
-
       vars_to_starts: A dictionary that maps variables to their global starting
         indices For an n-state variable, a global start index of m means the
         global indices of its n variable states are m, m + 1, ..., m + n - 1
-
       edge_states_offset: Offset to go from a variable's relevant state to its
-        other state For ORFactors the edge_states_offset is 1, for ANDFactors
-        the edge_states_offset is -1.
+        other state
+        For ORFactors the edge_states_offset is 1
+        For ANDFactors the edge_states_offset is -1
 
     Returns:
       The LogicalWiring
     """
-    var_states = []
-    for variables_for_factor in variables_for_factors:
+    # Step 1: compute var_states_for_edges
+    first_var_state_by_edges = []
+    factor_indices = []
+    for factor_idx, variables_for_factor in enumerate(variables_for_factors):
       for variable in variables_for_factor:
-        var_states.append(vars_to_starts[variable])
-    var_states = np.array(var_states)
+        first_var_state_by_edges.append(vars_to_starts[variable])
+        factor_indices.append(factor_idx)
+    first_var_state_by_edges = np.array(first_var_state_by_edges)
+    factor_indices = np.array(factor_indices)
 
-    # Relevant state differs for ANDFactors and ORFactors
-    relevant_state = (-edge_states_offset + 1) // 2
-
-    # Note: all the variables in a LogicalFactorGroup are binary
-    num_states_cumsum = np.arange(0, 2 * var_states.shape[0] + 2, 2)
-    var_states_for_edges = np.empty(shape=(2 * var_states.shape[0],), dtype=int)
-    factor.compile_var_states_numba(
-        var_states_for_edges, num_states_cumsum, var_states
+    # All the variables in a LogicalFactorGroup are binary
+    num_edges_states_cumsum = np.arange(
+        0, 2 * first_var_state_by_edges.shape[0] + 2, 2
+    )
+    var_states_for_edges = np.empty(
+        shape=(2 * first_var_state_by_edges.shape[0], 3), dtype=int
+    )
+    factor.compile_var_states_for_edges_numba(
+        var_states_for_edges,
+        num_edges_states_cumsum,
+        first_var_state_by_edges,
+        factor_indices,
     )
 
+    # Step 2: compute parents_edge_states and children_edge_states
     num_parents = factor_sizes - 1
     num_parents_cumsum = np.insert(np.cumsum(num_parents), 0, 0)
     parents_edge_states = np.empty(shape=(num_parents_cumsum[-1], 2), dtype=int)
     children_edge_states = np.empty(shape=(factor_sizes.shape[0],), dtype=int)
 
-    # Note: edges_num_states_cumsum corresponds to the factor_to_msgs_start
-    edges_num_states_cumsum = np.insert(np.cumsum(2 * factor_sizes), 0, 0)
+    # Relevant state differs for ANDFactors and ORFactors
+    relevant_state = (-edge_states_offset + 1) // 2
 
+    # Note: edges_num_states_cumsum corresponds to the factor_to_msgs_start
+    factor_num_edges_states_cumsum = np.insert(
+        np.cumsum(2 * factor_sizes), 0, 0
+    )
     _compile_logical_wiring_numba(
         parents_edge_states=parents_edge_states,
         children_edge_states=children_edge_states,
         num_parents=num_parents,
         num_parents_cumsum=num_parents_cumsum,
-        edges_num_states_cumsum=edges_num_states_cumsum,
+        factor_num_edges_states_cumsum=factor_num_edges_states_cumsum,
         relevant_state=relevant_state,
     )
 
     return LogicalWiring(
-        edges_num_states=factor_edges_num_states,
         var_states_for_edges=var_states_for_edges,
         parents_edge_states=parents_edge_states,
         children_edge_states=children_edge_states,
         edge_states_offset=edge_states_offset,
     )
 
+  @staticmethod
+  @jax.jit
+  def compute_energy(
+      edge_states_one_hot_decoding: jnp.ndarray,
+      parents_factor_indices: jnp.ndarray,
+      parents_msg_indices: jnp.ndarray,
+      children_edge_states: jnp.ndarray,
+      edge_states_offset: int,
+      log_potentials: Optional[jnp.ndarray] = None,
+  ) -> float:
+    """Returns the contribution to the energy of several LogicalFactors.
+
+    Args:
+      edge_states_one_hot_decoding: Array of shape (num_edge_states,)
+        Flattened array of one-hot decoding of the edge states connected to the
+        LogicalFactors
+
+      parents_factor_indices: Array of shape (num_parents,)
+        parents_factor_indices[ii] contains the global LogicalFactor index of
+        the parent variable's relevant state.
+        Only takes into account the LogicalFactors of the same subtype (OR/AND)
+        of the FactorGraph
+
+      parents_msg_indices: Array of shape (num_parents,)
+        parents_msg_indices[ii] contains the message index of the parent
+        variable's relevant state.
+        The message index of the parent variable's other state is
+        parents_msg_indices[ii] + edge_states_offset.
+        Only takes into account the LogicalFactors of the same subtype (OR/AND)
+        of the FactorGraph
+
+      children_edge_states: Array of shape (num_factors,)
+        children_edge_states[ii] contains the message index of the child
+        variable's relevant state
+        The message index of the child variable's other state is
+        children_edge_states[ii] + edge_states_offset
+        Only takes into account the LogicalFactors of the same subtype (OR/AND)
+        of the FactorGraph
+
+      edge_states_offset: Offset to go from a variable's relevant state to its
+        other state
+        For ORFactors the edge_states_offset is 1
+        For ANDFactors the edge_states_offset is -1
+
+      log_potentials: Optional array of log potentials
+    """
+    num_factors = children_edge_states.shape[0]
+
+    # parents_edge_states[..., 1] indicates the relevant state,
+    # which is 0 for ORFactors and 1 for ANDFactors
+    # If all the parents variables of a factor are in the factor type's relevant
+    # state, then the child variable must be in the relevant state too
+    logical_parents_decoded = (
+        jnp.ones(shape=(num_factors,))
+        .at[parents_factor_indices]
+        .multiply(edge_states_one_hot_decoding[parents_msg_indices])
+    )
+    logical_children_decoded = edge_states_one_hot_decoding[
+        children_edge_states
+    ]
+    energy = jnp.where(
+        jnp.any(logical_parents_decoded != logical_children_decoded),
+        jnp.inf,  # invalid decoding
+        0.0
+    )
+    return energy
+
 
 @dataclasses.dataclass(frozen=True, eq=False)
 class ORFactor(LogicalFactor):
@@ -275,11 +368,43 @@ class ORFactor(LogicalFactor):
 
   Attributes:
     edge_states_offset: Offset to go from a variable's relevant state to its
-      other state For ORFactors the edge_states_offset is 1.
+      other state
+      For ORFactors the edge_states_offset is 1.
   """
 
   edge_states_offset: int = dataclasses.field(init=False, default=1)
 
+  @staticmethod
+  def compute_factor_energy(
+      variables: List[Hashable],
+      vars_to_map_states: Dict[Hashable, Any],
+      **kwargs,
+  ) -> float:
+    """Returns the contribution to the energy of a single ORFactor.
+
+    Args:
+      variables: List of variables connected by the ORFactor
+      vars_to_map_states: A dictionary mapping each individual variable to
+        its MAP state.
+      **kwargs: Other parameters, not used
+    """
+    vars_decoded_states = np.array(
+        [vars_to_map_states[var] for var in variables]
+    )
+    parents_decoded_states = vars_decoded_states[:-1]
+    child_decoded_states = vars_decoded_states[-1]
+
+    if np.any(parents_decoded_states) != child_decoded_states:
+      warnings.warn(
+          f"Invalid decoding for OR factor {variables} "
+          f"with parents set to {parents_decoded_states} "
+          f"and child set to {child_decoded_states}!"
+      )
+      factor_energy = np.inf
+    else:
+      factor_energy = 0.0
+    return factor_energy
+
 
 @dataclasses.dataclass(frozen=True, eq=False)
 class ANDFactor(LogicalFactor):
@@ -291,11 +416,43 @@ class ANDFactor(LogicalFactor):
 
   Attributes:
     edge_states_offset: Offset to go from a variable's relevant state to its
-      other state For ANDFactors the edge_states_offset is -1.
+      other state
+      For ANDFactors the edge_states_offset is -1.
   """
 
   edge_states_offset: int = dataclasses.field(init=False, default=-1)
 
+  @staticmethod
+  def compute_factor_energy(
+      variables: List[Hashable],
+      vars_to_map_states: Dict[Hashable, Any],
+      **kwargs,
+  ) -> float:
+    """Returns the contribution to the energy of a single ANDFactor.
+
+    Args:
+      variables: List of variables connected by the ANDFactor
+      vars_to_map_states: A dictionary mapping each individual variable to
+        its MAP state.
+      **kwargs: Other parameters, not used
+    """
+    vars_decoded_states = np.array(
+        [vars_to_map_states[var] for var in variables]
+    )
+    parents_decoded_states = vars_decoded_states[:-1]
+    child_decoded_states = vars_decoded_states[-1]
+
+    if np.all(parents_decoded_states) != child_decoded_states:
+      warnings.warn(
+          f"Invalid decoding for AND factor {variables} "
+          f"with parents set to {parents_decoded_states} "
+          f"and child set to {child_decoded_states}!"
+      )
+      factor_energy = np.inf
+    else:
+      factor_energy = 0.0
+    return factor_energy
+
 
 # pylint: disable=g-doc-args
 @nb.jit(parallel=False, cache=True, fastmath=True, nopython=True)
@@ -304,7 +461,7 @@ def _compile_logical_wiring_numba(
     children_edge_states: np.ndarray,
     num_parents: np.ndarray,
     num_parents_cumsum: np.ndarray,
-    edges_num_states_cumsum: np.ndarray,
+    factor_num_edges_states_cumsum: np.ndarray,
     relevant_state: int,
 ):
   """Fast numba computation of the parents_edge_states and children_edge_states of a LogicalWiring.
@@ -319,12 +476,13 @@ def _compile_logical_wiring_numba(
     )
     parents_edge_states[start_parents:end_parents, 0] = factor_idx
     parents_edge_states[start_parents:end_parents, 1] = np.arange(
-        edges_num_states_cumsum[factor_idx] + relevant_state,
-        edges_num_states_cumsum[factor_idx] + 2 * num_parents[factor_idx],
+        factor_num_edges_states_cumsum[factor_idx] + relevant_state,
+        factor_num_edges_states_cumsum[factor_idx]
+        + 2 * num_parents[factor_idx],
         2,
     )
     children_edge_states[factor_idx] = (
-        edges_num_states_cumsum[factor_idx]
+        factor_num_edges_states_cumsum[factor_idx]
         + 2 * num_parents[factor_idx]
         + relevant_state
     )
@@ -370,7 +528,8 @@ def get_maxes_and_argmaxes(
 @functools.partial(jax.jit, static_argnames="temperature")
 def pass_logical_fac_to_var_messages(
     vtof_msgs: jnp.ndarray,
-    parents_edge_states: jnp.ndarray,
+    parents_factor_indices: jnp.ndarray,
+    parents_msg_indices: jnp.ndarray,
     children_edge_states: jnp.ndarray,
     edge_states_offset: int,
     temperature: float,
@@ -379,25 +538,30 @@ def pass_logical_fac_to_var_messages(
   """Passes messages from LogicalFactors to Variables.
 
   Args:
-    vtof_msgs: Array of shape (num_edge_state,). This holds all the
+    vtof_msgs: Array of shape (num_edge_states,). This holds all the
       flattened variable to all the LogicalFactors messages.
 
-    parents_edge_states: Array of shape (num_parents, 2)
-        parents_edge_states[ii, 0] contains the global LogicalFactor index
-        parents_edge_states[ii, 1] contains the message index of the parent
-        variable's relevant state
+    parents_factor_indices: Array of shape (num_parents,)
+      parents_factor_indices[ii] contains the global LogicalFactor index of the
+      parent variable's relevant state.
+      Only takes into account the LogicalFactors of the same subtype (OR/AND)
+      of the FactorGraph
 
-      Both indices only take into account the LogicalFactors of the same subtype
-      (OR/AND) of the FactorGraph
+    parents_msg_indices: Array of shape (num_parents,)
+      parents_msg_indices[ii] contains the message index of the parent
+      variable's relevant state.
       The message index of the parent variable's other state is
-      parents_edge_states[ii, 1] + edge_states_offset
+      parents_msg_indices[ii] + edge_states_offset.
+      Only takes into account the LogicalFactors of the same subtype (OR/AND)
+      of the FactorGraph
 
     children_edge_states: Array of shape (num_factors,)
       children_edge_states[ii] contains the message index of the child
-      variable's relevant state , which takes into account all the
-      LogicalFactors of the same subtype (OR/AND) of the FactorGraph
+      variable's relevant state
       The message index of the child variable's other state is
-      children_edge_states[ii, 1] + edge_states_offset
+      children_edge_states[ii] + edge_states_offset
+      Only takes into account the LogicalFactors of the same subtype (OR/AND)
+      of the FactorGraph
 
     edge_states_offset: Offset to go from a variable's relevant state to its
       other state
@@ -410,15 +574,14 @@ def pass_logical_fac_to_var_messages(
     log_potentials: Optional array of log potentials
 
   Returns:
-      Array of shape (num_edge_state,). This holds all the flattened ORFactors
-      to variable messages.
+    Array of shape (num_edge_states,). This holds all the flattened
+    LogicalFactors of the same subtype (OR/AND) to variable messages.
   """
   num_factors = children_edge_states.shape[0]
-  factor_indices = parents_edge_states[..., 0]
 
   parents_tof_msgs = (
-      vtof_msgs[parents_edge_states[..., 1] + edge_states_offset]
-      - vtof_msgs[parents_edge_states[..., 1]]
+      vtof_msgs[parents_msg_indices + edge_states_offset]
+      - vtof_msgs[parents_msg_indices]
   )
   children_tof_msgs = (
       vtof_msgs[children_edge_states + edge_states_offset]
@@ -430,28 +593,29 @@ def pass_logical_fac_to_var_messages(
   if temperature == 0.0:
     # Get first and second argmaxes for incoming parents messages of each factor
     _, first_parents_argmaxes = get_maxes_and_argmaxes(
-        parents_tof_msgs, factor_indices, num_factors
+        parents_tof_msgs, parents_factor_indices, num_factors
     )
     _, second_parents_argmaxes = get_maxes_and_argmaxes(
         parents_tof_msgs.at[first_parents_argmaxes].set(NEG_INF),
-        factor_indices,
+        parents_factor_indices,
         num_factors,
     )
 
     parents_tof_msgs_pos = jnp.maximum(0.0, parents_tof_msgs)
     sum_parents_tof_msgs_pos = (
         jnp.full(shape=(num_factors,), fill_value=0.0)
-        .at[factor_indices]
+        .at[parents_factor_indices]
         .add(parents_tof_msgs_pos)
     )
 
     # Outgoing messages to parents variables
     parents_msgs = jnp.minimum(
-        children_tof_msgs[factor_indices]
-        + sum_parents_tof_msgs_pos[factor_indices]
+        children_tof_msgs[parents_factor_indices]
+        + sum_parents_tof_msgs_pos[parents_factor_indices]
         - parents_tof_msgs_pos,
         jnp.maximum(
-            0.0, -parents_tof_msgs[first_parents_argmaxes][factor_indices]
+            0.0,
+            -parents_tof_msgs[first_parents_argmaxes][parents_factor_indices],
         ),
     )
     parents_msgs = parents_msgs.at[first_parents_argmaxes].set(
@@ -489,26 +653,27 @@ def pass_logical_fac_to_var_messages(
     )
     sum_log_sig_parents_tof_msgs = (
         jnp.full(shape=(num_factors,), fill_value=0.0)
-        .at[factor_indices]
+        .at[parents_factor_indices]
         .add(log_sig_parents_tof_msgs)
     )
 
     g_sum_log_sig_parents_minus_id = g(
-        sum_log_sig_parents_tof_msgs[factor_indices] - log_sig_parents_tof_msgs
+        sum_log_sig_parents_tof_msgs[parents_factor_indices]
+        - log_sig_parents_tof_msgs
     )
 
     # Outgoing messages to parents variables
     parents_msgs = -temperature * jnp.log(
         sigmoid(g_sum_log_sig_parents_minus_id / temperature)
         + sigmoid(-g_sum_log_sig_parents_minus_id / temperature)
-        * jnp.exp(-children_tof_msgs[factor_indices] / temperature)
+        * jnp.exp(-children_tof_msgs[parents_factor_indices] / temperature)
     )
 
     # Outgoing messages to children variables
     children_msgs = g(sum_log_sig_parents_tof_msgs)
 
   # Special case: factors with a single parent
-  num_parents = jnp.bincount(factor_indices, length=num_factors)
+  num_parents = jnp.bincount(parents_factor_indices, length=num_factors)
   first_elements = jnp.concatenate(
       [jnp.zeros(1, dtype=int), jnp.cumsum(num_parents)]
   )[:-1]
@@ -519,9 +684,9 @@ def pass_logical_fac_to_var_messages(
   )
 
   ftov_msgs = jnp.zeros_like(vtof_msgs)
-  ftov_msgs = ftov_msgs.at[
-      parents_edge_states[..., 1] + edge_states_offset
-  ].set(parents_msgs)
+  ftov_msgs = ftov_msgs.at[parents_msg_indices + edge_states_offset].set(
+      parents_msgs
+  )
   ftov_msgs = ftov_msgs.at[children_edge_states + edge_states_offset].set(
       children_msgs
   )
