@@ -1,6 +1,5 @@
-# pyformat: mode=midnight
-# ==============================================================================
 # Copyright 2022 Intrinsic Innovation LLC.
+# Copyright 2022 DeepMind Technologies Limited.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-# ==============================================================================
+
 """A module containing the core class to build a factor graph."""
 
 import collections
@@ -50,6 +49,7 @@ class FactorGraphState:
     factor_group_to_potentials_starts: Maps factor groups to their starting
       indices in the flat log potentials.
     log_potentials: Flat log potentials array concatenated for each factor type.
+    evidence_to_vars: Maps the evidence entries to their variable indices
     wiring: Wiring derived for each factor type.
   """
 
@@ -63,6 +63,7 @@ class FactorGraphState:
   ]
   factor_group_to_potentials_starts: OrderedDict[fgroup.FactorGroup, int]
   log_potentials: np.ndarray
+  evidence_to_vars: np.ndarray
   wiring: OrderedDict[Type[factor.Factor], factor.Wiring]
 
   def __post_init__(self):
@@ -205,12 +206,12 @@ class FactorGraph:
       factor_type_num_states_start = factor_num_states_cumsum
       factor_type_num_configs_start = factor_num_configs_cumsum
       for factor_group in factors_groups_by_type:
-        self._factor_group_to_msgs_starts[
-            factor_group
-        ] = factor_num_states_cumsum
-        self._factor_group_to_potentials_starts[
-            factor_group
-        ] = factor_num_configs_cumsum
+        self._factor_group_to_msgs_starts[factor_group] = (
+            factor_num_states_cumsum
+        )
+        self._factor_group_to_potentials_starts[factor_group] = (
+            factor_num_configs_cumsum
+        )
 
         factor_num_states_cumsum += factor_group.factor_edges_num_states.sum()
         factor_num_configs_cumsum += (
@@ -312,6 +313,17 @@ class FactorGraph:
     """Tuple of factor groups in the factor graph."""
     return self._factor_types_to_groups
 
+  @property
+  def evidence_to_vars(self) -> np.ndarray:
+    """Returns the variable index associated with each evidence entry."""
+    var_indices = list(self._vars_to_starts.values()) + [self._num_var_states]
+
+    evidence_to_vars = np.zeros((self._num_var_states,))
+    for idx_var in range(len(var_indices) - 1):
+      var_start, var_stop = var_indices[idx_var], var_indices[idx_var + 1]
+      evidence_to_vars[var_start:var_stop] = idx_var
+    return evidence_to_vars
+
   @cached_property
   def fg_state(self) -> FactorGraphState:
     """Current factor graph state given the added factors."""
@@ -338,6 +350,7 @@ class FactorGraph:
             self._factor_group_to_potentials_starts
         ),
         log_potentials=log_potentials,
+        evidence_to_vars=self.evidence_to_vars,
         wiring=self.wiring,
     )
 
