@@ -1,6 +1,5 @@
-# pyformat: mode=midnight
-# ==============================================================================
 # Copyright 2022 Intrinsic Innovation LLC.
+# Copyright 2023 DeepMind Technologies Limited.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,9 +12,10 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-# ==============================================================================
+
 """A sub-package containing functions to perform belief propagation."""
 
+from pgmax.infer.bp import BeliefPropagation
 from pgmax.infer.bp import BP
 from pgmax.infer.bp import decode_map_states
 from pgmax.infer.bp import get_marginals
@@ -24,3 +24,4 @@ from pgmax.infer.bp_state import BPState
 from pgmax.infer.bp_state import Evidence
 from pgmax.infer.bp_state import FToVMessages
 from pgmax.infer.bp_state import LogPotentials
+from pgmax.infer.energy import compute_energy
